import _toConsumableArray from 'babel-runtime/helpers/toConsumableArray';
import _extends from 'babel-runtime/helpers/extends';
import '../assets/propsSetter.less';
import '../assets/columnsSetterModal.less';
import Vue from 'vue';
import { Button, Modal, Input, Row, Col, Checkbox, Tabs } from 'ant-design-vue';
import LocalStorage from '../utils/localStorage';
import { debounce, transformArrToMap } from '../utils/tools';
Vue.use(Button).use(Modal).use(Input).use(Row).use(Col).use(Checkbox).use(Tabs);

var ColumnsSetterModal = {
  name: "ColumnsSetterModal",
  components: {
    // Button,
    // Modal,
    // Input,
    // Row,
    // Col,
    // Checkbox,
    // Tabs
  },
  props: {
    propList: Array,
    localStorageKey: String
    // pageName: String, // 当前页面的名称，用户保存当前选择的列数据
    // columnsConfig: Array, // 初始化列的数据
    // dataType: String, // 报表类型
    // checkPermission: {// 是否需要判断权限
    //   type: Boolean,
    //   default: true
    // }
  },
  data: function data() {
    return {
      visible: false, // 是否显示弹窗
      keyWord: '', // 搜索关键词
      filtering: false,
      filterList: [], // 搜索结果列表
      valueMap: {}, // 不同tab下选中的指标值
      valueMapJSONStringfy: '',
      validColumnsConfig: [], // 有效的指标
      /// new
      validColumnsGroup: [],
      groupValueData: {},
      localStorageHandler: null // 每个group的值
    };
  },

  computed: {
    columnMap: function columnMap() {
      var count = 0;
      return this.validColumnsGroup.reduce(function (map, _ref) {
        var groupKey = _ref.groupKey,
            list = _ref.list;

        list && list.forEach(function (col) {
          map[col.dataIndex] = _extends({}, col, {
            index: col.index || count++,
            groupKey: groupKey,
            disabled: col.disabled || !!col.fixed
          });
        });
        return map;
      }, {});
    },
    columnsTypeValuesMap: function columnsTypeValuesMap() {
      // 记录每个group必选的和全部的数据数量
      return this.validColumnsGroup.reduce(function (map, _ref2) {
        var groupKey = _ref2.groupKey,
            list = _ref2.list;

        var fullValues = [];
        var requiredValues = [];
        list.forEach(function (_ref3) {
          var dataIndex = _ref3.dataIndex,
              disabled = _ref3.disabled;

          fullValues.push(dataIndex);
          if (disabled) {
            requiredValues.push(dataIndex);
          }
        });
        map[groupKey] = {
          fullValues: fullValues,
          requiredValues: requiredValues
        };
        return map;
      }, {});
    },

    // defaultTab
    defaultTab: function defaultTab() {
      // 默认选中的tab
      return this.validColumnsGroup[0] && this.validColumnsGroup[0].groupKey;
    },
    checkedColumns: function checkedColumns() {
      var _this = this;

      return Object.values(this.groupValueData).reduce(function (res, _ref4) {
        var checkedKeys = _ref4.checkedKeys;

        res = res.concat(checkedKeys.map(function (key) {
          return _this.columnMap[key];
        }));
        return res;
      }, []);
    },
    checkedPropMap: function checkedPropMap() {
      return this.checkedColumns.reduce(function (res, item) {
        res[item.dataIndex] = true;
        return res;
      }, {});
    },
    noOpProps: function noOpProps() {
      return this.propList.filter(function (item) {
        return item.noOp;
      });
    }
  },
  watch: {
    visible: function visible(val) {
      // if (!val) return
      // this.init()
      // this.valueMapJSONStringfy = JSON.stringify(this.valueMap)
    },

    // colPermissionMap (map) {
    //   if (!this.checkPermission) return
    //   this.validColumnsConfig = this.setValidColumns(map)
    //   this.init()
    // },
    // checkPermission: {
    //   immediate: true,
    //   handler (needCheck) {
    //     if (!needCheck) {
    //       // this.validColumnsConfig = this.columnsConfig
    //       // this.init()
    //       let { props } = this.permissionConfig || {}
    //       this.initColumnsGroup(props || [])
    //     }
    //   }
    // },
    // new
    propList: {
      immediate: true,
      handler: function handler(list) {
        // let { props } = this.permissionConfig
        this.initColumnsGroup(list);
        // this.init()
      }
    }
  },
  mounted: function mounted() {
    this.debounceFn = debounce(500, this.handleColumnsSearch);
    this.useLocalStorage();
  },

  methods: {
    initColumnsGroup: function initColumnsGroup(list) {
      var map = {};
      var count = 0;
      list.forEach(function (item) {
        var _item$group = item.group,
            groupKey = _item$group.key,
            groupName = _item$group.name;

        if (!map[groupKey]) {
          count++;
          map[groupKey] = {
            groupKey: groupKey,
            groupName: groupName,
            list: [],
            index: count
          };
        }
        // 去除不在自定义中展示的列
        !item.noOp && map[groupKey].list.push(item);
      });
      this.validColumnsGroup = Object.values(map).sort(function (pre, cur) {
        return pre.index - cur.index;
      });
      this.initGroupChecked();
    },
    useLocalStorage: function useLocalStorage() {
      if (this.localStorageHandler) return;
      this.localStorageHandler = new LocalStorage(this.localStorageKey);
    },
    initGroupChecked: function initGroupChecked() {
      // 初始化已选的默认值
      // 判断local storage中是否存有记录
      this.useLocalStorage();
      var localStorageCheckedKeys = [];
      try {
        localStorageCheckedKeys = JSON.parse(this.localStorageHandler.getValue());
        if (!Array.isArray(localStorageCheckedKeys)) {
          // eslint-disable-next-line no-throw-literal
          throw 'local storage所存数据有误，现使用默认数据';
        }
      } catch (error) {
        console.error(error || '读取local storage数据失败，现使用默认数据');
      }

      var localStorageCheckedMap = localStorageCheckedKeys.length > 0 ? transformArrToMap(localStorageCheckedKeys) : null;

      this.groupValueData = this.validColumnsGroup.reduce(function (map, _ref5) {
        var groupKey = _ref5.groupKey,
            list = _ref5.list;

        // 筛选已选值
        var checkedKeys = list.filter(function (item) {
          var dataIndex = item.dataIndex,
              disabled = item.disabled,
              isDefault = item.isDefault;

          if (disabled) return true; // disabled强制选中
          if (localStorageCheckedMap) {
            // 如果local storage中存了，则使用存的数据判断
            return !!localStorageCheckedMap[dataIndex];
          }
          // 否则使用默认值
          return isDefault;
        }).map(function (_ref6) {
          var dataIndex = _ref6.dataIndex;
          return dataIndex;
        });

        var checkedAll = checkedKeys.length === list.length; // 是否全选
        map[groupKey] = {
          indeterminate: !checkedAll && checkedKeys.length > 0,
          checkedAll: checkedAll,
          checkedKeys: checkedKeys
        };
        return map;
      }, {});
      this.emitChange();
    },
    handleApply: function handleApply() {
      var _this2 = this;

      // 保存 触发父组件数据更新，并存储到local storage中
      this.emitChange(function (keys) {
        if (!_this2.localStorageKey) {
          console.error('未传入localStorageKey，无法实现自定义列保存，请检查');
          return;
        }
        _this2.localStorageHandler.setValue(keys);
        _this2.visible = false;
        _this2.keyWord = '';
      });
    },
    close: function close() {
      this.visible = false;
      this.keyWord = '';
      this.recoverySelect();
    },
    onSearch: function onSearch(e) {
      this.keyWord = e.target ? e.target.value : e;
      this.filtering = true;
      this.debounceFn();
    },
    handleSearchItemChecked: function handleSearchItemChecked(e, _ref7) {
      var groupKey = _ref7.groupKey,
          dataIndex = _ref7.dataIndex;
      // 处理筛选项选中事件
      var checked = e.target.checked;
      if (checked) {
        this.groupValueData[groupKey].checkedKeys.push(dataIndex);
      } else {
        this.groupValueData[groupKey].checkedKeys = this.groupValueData[groupKey].checkedKeys.filter(function (key) {
          return key !== dataIndex;
        });
      }
      var checkedCount = this.groupValueData[groupKey].checkedKeys.length;
      this.groupValueData[groupKey].checkedAll = checkedCount === this.columnsTypeValuesMap[groupKey].fullValues.length;
      this.groupValueData[groupKey].indeterminate = !this.groupValueData[groupKey].checkedAll && checkedCount > 0;
      // this.updateChecked()
    },
    handleColumnsSearch: function handleColumnsSearch() {
      var _this3 = this;

      this.filterList = Object.values(this.columnMap).filter(function (_ref8) {
        var title = _ref8.title,
            disabled = _ref8.disabled;

        // 禁用指标不参与筛选展示
        return title.includes(_this3.keyWord) && !disabled;
      });
      console.log(223, this.filterList);
      this.filtering = false;
    },

    // 恢复初始(即未保存前的状态值)
    recoverySelect: function recoverySelect() {
      this.initGroupChecked();
    },
    onSelectAllChange: function onSelectAllChange(e, groupKey) {
      // 处理tab全选变动事件
      var checked = e.target.checked;
      this.groupValueData[groupKey].checkedAll = checked;
      if (checked) {
        this.groupValueData[groupKey].checkedKeys = this.columnsTypeValuesMap[groupKey].fullValues;
        this.groupValueData[groupKey].indeterminate = false;
      } else {
        // 取消全选 需要保留disabled指标
        this.groupValueData[groupKey].checkedKeys = this.columnsTypeValuesMap[groupKey].requiredValues;
        this.groupValueData[groupKey].indeterminate = this.groupValueData[groupKey].checkedKeys.length > 0;
      }
      this.updateChecked();
    },
    deleteSelectColunm: function deleteSelectColunm(key, groupKey) {
      var selectColumnKeys = this.groupValueData[groupKey].checkedKeys.filter(function (k) {
        return k !== key;
      });
      this.groupValueData[groupKey].checkedKeys = selectColumnKeys;
      this.groupValueData[groupKey].checkedAll = false;
      this.groupValueData[groupKey].indeterminate = selectColumnKeys.length > 0;
      this.updateChecked();
    },
    updateChecked: function updateChecked(updateType) {// 传入updateType,则计算对应valueMap的indeterminate和isSelectAll值
      // let columns = []
      // let keys = []
      // Object.entries(this.valueMap).forEach(([type, { selectColumnKeys }]) => {
      //   // 通过key读取对应完整配置
      //   selectColumnKeys.forEach(key => {
      //     let data = this.columnMap[key]
      //     // if (Number(type) !== 1) { // 非基础指标可排序
      //     //   data.sortable = true
      //     // }
      //     data && columns.push(data)
      //   })
      //   keys = keys.concat(selectColumnKeys)
      // })
      // // 处理排序，保证选中的数据按照分类及在分类中的先后次序展示在列表中
      // columns = columns.sort((pre, cur) => pre.index - cur.index)
      // // this.checkedColumns = columns

      // // 处理指定type的indeterminate和isSelectAll值
      // if (updateType && this.valueMap[updateType]) {
      //   this.valueMap[updateType].isSelectAll = this.valueMap[updateType].selectColumnKeys.length === this.columnsTypeValuesMap[updateType].fullValues.length
      //   this.valueMap[updateType].indeterminate = !this.valueMap[updateType].isSelectAll && this.valueMap[updateType].selectColumnKeys.length > 0
      // }
      // return keys
    },
    updateCheckedKey: function updateCheckedKey(groupKey, keys) {
      var checkedCount = keys.length;
      var checkedAll = checkedCount === this.columnsTypeValuesMap[groupKey].fullValues.length;
      this.groupValueData[groupKey].checkedAll = checkedAll;
      this.groupValueData[groupKey].indeterminate = !checkedAll && checkedCount > 0;
      this.groupValueData[groupKey].checkedKeys = keys;
    },
    getCheckedKeys: function getCheckedKeys(checkedItems) {
      return checkedItems.map(function (_ref9) {
        var dataIndex = _ref9.dataIndex;
        return dataIndex;
      });
    },
    emitChange: function emitChange(callback) {
      var colList = this.formatColOrder();
      this.$emit('changeCoumns', colList);
      // 存在回调
      callback && callback(this.getCheckedKeys(this.checkedColumns));
    },
    formatColOrder: function formatColOrder() {
      var _reduce = (this.noOpProps || []).reduce(function (res, item) {
        if (item.position === 'right') {
          res.rightProps.push(item);
        } else {
          res.leftProps.push(item);
        }
        return res;
      }, {
        leftProps: [],
        rightProps: []
      }),
          leftProps = _reduce.leftProps,
          rightProps = _reduce.rightProps;

      var _checkedColumns$reduc = this.checkedColumns.reduce(function (res, item) {
        if (item.fixed) {
          if (item.fixed === 'right') {
            res.fixedRightProps.push(item);
          } else {
            res.fixedLeftProps.push(item);
          }
        } else {
          res.others.push(item);
        }
        return res;
      }, {
        fixedLeftProps: [],
        fixedRightProps: [],
        others: []
      }),
          fixedLeftProps = _checkedColumns$reduc.fixedLeftProps,
          fixedRightProps = _checkedColumns$reduc.fixedRightProps,
          others = _checkedColumns$reduc.others;

      return [].concat(_toConsumableArray(fixedLeftProps), _toConsumableArray(leftProps), _toConsumableArray(others), _toConsumableArray(rightProps), _toConsumableArray(fixedRightProps));
    },
    renderTab: function renderTab(groupKey, list) {
      var _this4 = this;

      var h = this.$createElement;

      if (!this.groupValueData[groupKey]) return;

      var renderContent = function renderContent(list) {
        if (list && list.length > 0) {
          return h('a-row', [h(
            'a-col',
            {
              attrs: { span: 24 },
              'class': 'col-item' },
            [h(
              'a-checkbox',
              {
                attrs: {
                  indeterminate: _this4.groupValueData[groupKey].indeterminate,
                  checked: _this4.groupValueData[groupKey].checkedAll
                },
                on: {
                  'change': function change(e) {
                    return _this4.onSelectAllChange(e, groupKey);
                  }
                }
              },
              ['\u5168\u90E8']
            )]
          )]);
        } else {
          return h(
            'a-row',
            { 'class': 'empty-txt' },
            ['\u6682\u65E0\u53EF\u9009\u6307\u6807']
          );
        }
      };
      return h(
        'div',
        { 'class': 'checkbox-list-wrapper' },
        [this.renderContent(list), h(
          'a-checkbox-group',
          {
            attrs: { value: this.groupValueData[groupKey].checkedKeys
            },
            on: {
              'change': function change(v) {
                return updateCheckedKey(groupKey, v);
              }
            }
          },
          [h('a-row', [list.map(function (item) {
            return h(
              'a-col',
              { key: item.dataIndex, attrs: { span: 12 },
                'class': 'col-item' },
              [h(
                'a-checkbox',
                {
                  attrs: { value: item.dataIndex, disabled: item.disabled }
                },
                [h(
                  'span',
                  {
                    attrs: { title: item.title },
                    'class': 'ellipsis-label' },
                  [item.title]
                )]
              )]
            );
          })])]
        )]
      );
    }
  },
  render: function render() {
    var _this5 = this;

    var h = arguments[0];

    return h('template', [h('div', [h(
      'span',
      {
        attrs: { onlick: function onlick() {
            _this5.visible = true;
          } }
      },
      [h('slot', [h('a-button', ['\u81EA\u5B9A\u4E49\u5217'])])]
    ), h(
      'a-modal',
      {
        attrs: { visible: this.visible, title: '\u81EA\u5B9A\u4E49\u5217', width: '1000px', maskClosable: false },
        on: {
          'cancel': this.close
        }
      },
      [h(
        'div',
        { 'class': 'search-panel' },
        [h('a-input-search', {
          attrs: { value: this.keyWord, allowClear: true, placeholder: '\u641C\u7D22\u5217\u540D' },
          style: 'width:260px;', on: {
            'change': this.onSearch
          }
        })]
      ), h(
        'div',
        { 'class': 'custom-report-columns columns-action-panel' },
        [h(
          'div',
          { 'class': 'left-contents' },
          [h(
            'div',
            {
              directives: [{
                name: 'show',
                value: 'keyWord'
              }]
            },
            [h(
              'a-row',
              {
                directives: [{
                  name: 'show',
                  value: 'filterList.length > 0'
                }]
              },
              [this.filterList.map(function (_ref10) {
                var title = _ref10.title,
                    dataIndex = _ref10.dataIndex,
                    groupKey = _ref10.groupKey,
                    disabled = _ref10.disabled;

                return h(
                  'a-col',
                  { key: dataIndex, attrs: { span: 12 },
                    'class': 'col-item' },
                  [h(
                    'a-checkbox',
                    {
                      attrs: { value: dataIndex, checked: _this5.checkedPropMap[dataIndex],
                        disabled: disabled
                      },
                      on: {
                        'change': function change(e) {
                          return _this5.handleSearchItemChecked(e, { dataIndex: dataIndex, groupKey: groupKey });
                        }
                      }
                    },
                    [title]
                  )]
                );
              })]
            ), h(
              'p',
              {
                directives: [{
                  name: 'show',
                  value: 'filterList.length < 1'
                }],
                'class': 'empty-list' },
              [this.filtering ? '数据匹配中...' : '未匹配到数据']
            )]
          ), h(
            'a-tabs',
            {
              directives: [{
                name: 'show',
                value: '!keyWord'
              }],
              attrs: { 'default-active-key': this.defaultTab, size: 'small', tabBarStyle: { padding: 0 } }
            },
            [this.validColumnsGroup.map(function (_ref11) {
              var groupKey = _ref11.groupKey,
                  groupName = _ref11.groupName,
                  list = _ref11.list;

              return h(
                'a-tabPane',
                { key: groupKey, attrs: { tab: groupName }
                },
                [_this5.renderTab()]
              );
            })]
          )]
        ), h(
          'div',
          { 'class': 'right-contents' },
          [h(
            'div',
            { 'class': 'header-row' },
            [h(
              'div',
              { 'class': 'label-text' },
              ['\u5DF2\u9009\u6307\u6807(', this.checkedColumns.length, ')']
            ), h(
              'div',
              { 'class': 'label-text active', on: {
                  'click': this.recoverySelect
                }
              },
              ['\u6062\u590D\u521D\u59CB']
            )]
          ), h(
            'div',
            { 'class': 'inner-row' },
            [this.checkedColumns.map(function (_ref12) {
              var dataIndex = _ref12.dataIndex,
                  title = _ref12.title,
                  disabled = _ref12.disabled,
                  groupKey = _ref12.groupKey;

              return h(
                'div',
                { 'class': 'select-column-info', key: dataIndex },
                [h(
                  'div',
                  { 'class': 'title-text', attrs: { title: title }
                  },
                  [title]
                ), h(
                  'a-button',
                  {
                    attrs: { type: 'link' },
                    directives: [{
                      name: 'show',
                      value: '!disabled'
                    }],
                    on: {
                      'click': _this5.deleteSelectColunm(dataIndex, groupKey)
                    },
                    'class': 'c-delete-btn' },
                  [h('span', { 'class': 'delete-icon' })]
                )]
              );
            })]
          )]
        )]
      ), h(
        'template',
        { slot: 'footer' },
        [h(
          'a-button',
          { key: 'back', on: {
              'click': this.close
            }
          },
          ['\u53D6\u6D88']
        ), h(
          'a-button',
          { key: 'submit', attrs: { type: 'primary' },
            on: {
              'click': this.handleApply
            }
          },
          ['\u5E94\u7528']
        )]
      )]
    )])]);
  }
};

/* istanbul ignore next */
ColumnsSetterModal.install = function (Vue) {
  // Vue.use(Base);
  Vue.component(ColumnsSetterModal.name, ColumnsSetterModal);
};

export default ColumnsSetterModal;